<?php
/**
* Provides various info about the current device,
* caches values locally
*/
class DuneBox
{
	private static $mac_address = null;
	private static $serial_number = null;
	private static $firmware_version = null;
	private static $product_id = null;


	/**
	 * Get MAC address
	 * @return string $mac_address MAC address
	 */
	public static function get_mac_addr()
	{
		$mac_addr = self::$mac_address;
		if (is_null(self::$mac_address))
		{
			$mac_addr = shell_exec(
				'ifconfig  eth0 | head -1 | sed "s/^.*HWaddr //"');
			$mac_addr = trim($mac_addr);
			self::$mac_address = $mac_addr;
			hd_print("MAC Address: '$mac_addr'");
		}
		return $mac_addr;
	}

	/**
	 * Get serial number
	 * @return string $serial_number Serial number
	 */
	public static function get_serial_number()
	{
		//$v = trim( shell_exec('grep \'serial_number\' /tmp/sysinfo.txt | sed "s/^.*:*//"') );
		//hd_print("Serial Number: '$v'");

		if (is_null(self::$serial_number))
		{
			$arr = file('/tmp/sysinfo.txt'); $v = '';
			foreach($arr as $line)
			{
				if ( stristr($line, 'serial_number: ') ) {
					$v = trim( substr($line, 15) );
				}
			}
			self::$serial_number = $v;
			hd_print("Serial Number: '$v'");
		}

		return self::$serial_number;
	}

	/**
	 * Get firmware version
	 * @return string $firmware_version Firmware version
	 */
	public static function get_firmware_version()
	{
		if (is_null(self::$firmware_version))
		{
			$arr = file('/tmp/sysinfo.txt'); $v = '';
			foreach($arr as $line)
			{
				if ( stristr($line, 'firmware_version: ') ) {
					$v = trim( substr($line, 18) );
				}
			}
			self::$firmware_version = $v;
			hd_print("Firmware Version: '$v'");
		}

		return self::$firmware_version;
	}

	/**
	 * Get product id
	 * @return string $product_id Product ID
	 */
	public static function get_product_id()
	{
		if (is_null(self::$product_id))
		{
			$arr = file('/tmp/sysinfo.txt'); $v = '';
			foreach($arr as $line)
			{
				if ( stristr($line, 'product_id: ') )
				{
					$v = trim( substr($line, 12) );
				}
			}
			self::$product_id = $v;
			hd_print("Product id: '$v'");
		}
		return self::$product_id;
	}

}

