<?php
/**
 * @author Sergey Jdanuk <jdanuk at gmail.com>
 */
class Rise {
    const VERSION = '1.0';
    const CODENAME = 'Alpha';
    const PRODUCTION = 10;
    const STAGING = 20;
    const TESTING = 30;
    const DEVELOPMENT = 40;
    public static $environment = Rise::PRODUCTION;
    public static $is_cli = FALSE;
    public static $is_windows = FALSE;
    public static $magic_quotes = FALSE;
    public static $log_errors = FALSE;
    public static $safe_mode = FALSE;
    public static $content_type = 'text/html';
    public static $charset = 'utf-8';
    public static $server_name = '';
    public static $hostnames = array();
    public static $base_url = '/';
    public static $index_file = 'index.php';
    public static $cache_dir;
    public static $cache_life = 60;
    public static $caching = FALSE;
    public static $errors = TRUE;
    public static $shutdown_errors = array(E_PARSE, E_ERROR, E_USER_ERROR);
    public static $expose = FALSE;
    public static $log;
    public static $config;
    protected static $_init = FALSE;
    protected static $_modules = array();
    protected static $_paths = array(APPPATH, SYSPATH);
    protected static $_files = array();
    protected static $_files_changed = FALSE;

    public static function init(array $settings = NULL) {
        register_shutdown_function(array('Rise', 'shutdown_handler'));
        Rise::$config = new Config;
    }

    public static function deinit() {
        if (Rise::$_init) {
            spl_autoload_unregister(array('Rise', 'auto_load'));

            if (Rise::$errors) {
                restore_error_handler();
                restore_exception_handler();
            }
            Rise::$log = Rise::$config = NULL;
            Rise::$_modules = Rise::$_files = array();
            Rise::$_paths = array(APPPATH, SYSPATH);
            Rise::$_files_changed = FALSE;
            Rise::$_init = FALSE;
        }
    }

    public static function sanitize($value) {
        if (is_array($value) OR is_object($value)) {
            foreach ($value as $key => $val) {
                $value[$key] = Rise::sanitize($val);
            }
        } elseif (is_string($value)) {
            if (Rise::$magic_quotes === TRUE) {
                $value = stripslashes($value);
            }

            if (strpos($value, "\r") !== FALSE) {
                $value = str_replace(array("\r\n", "\r"), "\n", $value);
            }
        }

        return $value;
    }

    public static function auto_load($class) {
        $file = str_replace('_', '/', strtolower($class));

        if ($path = Rise::find_file('classes', $file)) {
            require $path;
            return TRUE;
        }
        return FALSE;
    }

    public static function modules(array $modules = NULL) {
        if ($modules === NULL) {
            return Rise::$_modules;
        }
        $paths = array(APPPATH);

        foreach ($modules as $name => $path) {
            if (is_dir($path)) {
                $paths[] = $modules[$name] = realpath($path) . DIRECTORY_SEPARATOR;
            } else {
                throw new Exception('Attempted to load an invalid or missing module \':module\' at \':path\'', array(
                    ':module' => $name,
                    ':path' => $path,
                ));
            }
        }

        $paths[] = SYSPATH;
        Rise::$_paths = $paths;
        Rise::$_modules = $modules;
        foreach (Rise::$_modules as $path) {
            $init = $path . 'init' . EXT;
            if (is_file($init)) {
                require_once $init;
            }
        }

        return Rise::$_modules;
    }

    public static function include_paths() {
        return Rise::$_paths;
    }

    public static function find_file($dir, $file, $ext = NULL, $array = FALSE) {
        if ($ext === NULL) {
            $ext = EXT;
        } elseif ($ext) {
            $ext = ".{$ext}";
        } else {
            $ext = '';
        }
        $path = sprintf("%s%s%s%s", $dir, DIRECTORY_SEPARATOR, $file, $ext);

        if (Rise::$caching === TRUE AND isset(Rise::$_files[$path . ($array ? '_array' : '_path')])) {
            return Rise::$_files[$path . ($array ? '_array' : '_path')];
        }

        if ($array OR $dir === 'config' OR $dir === 'i18n' OR $dir === 'messages') {
            $paths = array_reverse(Rise::$_paths);
            $found = array();
            foreach ($paths as $dir) {
                if (is_file($dir . $path)) {
                    $found[] = $dir . $path;
                }
            }
        } else {
            $found = FALSE;
            foreach (Rise::$_paths as $dir) {
                if (is_file($dir . $path)) {
                    $found = $dir . $path;
                    break;
                }
            }
        }

        if (Rise::$caching === TRUE) {
            Rise::$_files[$path . ($array ? '_array' : '_path')] = $found;
            Rise::$_files_changed = TRUE;
        }

        return $found;
    }

    public static function list_files($directory = NULL, array $paths = NULL) {
        if ($directory !== NULL) {
            $directory .= DIRECTORY_SEPARATOR;
        }

        if ($paths === NULL) {
            $paths = Rise::$_paths;
        }
        $found = array();
        foreach ($paths as $path) {
            if (is_dir($path . $directory)) {
                $dir = new DirectoryIterator($path . $directory);

                foreach ($dir as $file) {
                    $filename = $file->getFilename();

                    if ($filename[0] === '.' OR $filename[strlen($filename) - 1] === '~') {
                        continue;
                    }

                    $key = $directory . $filename;

                    if ($file->isDir()) {
                        if ($sub_dir = Rise::list_files($key, $paths)) {
                            if (isset($found[$key])) {
                                $found[$key] += $sub_dir;
                            } else {
                                $found[$key] = $sub_dir;
                            }
                        }
                    } else {
                        if (!isset($found[$key])) {
                            $found[$key] = realpath($file->getPathName());
                        }
                    }
                }
            }
        }
        ksort($found);
        return $found;
    }

    public static function load($file) {
        return include $file;
    }

    public static function cache($name, $data = NULL, $lifetime = NULL) {
        $file = sha1($name) . '.txt';
        $dir = Rise::$cache_dir . DIRECTORY_SEPARATOR . $file[0] . $file[1] . DIRECTORY_SEPARATOR;
        if ($lifetime === NULL) {
            $lifetime = Rise::$cache_life;
        }

        if ($data === NULL) {
            if (is_file($dir . $file)) {
                if ((time() - filemtime($dir . $file)) < $lifetime) {
                    try {
                        return unserialize(file_get_contents($dir . $file));
                    } catch (Exception $e) {
                    }
                } else {
                    try {
                        unlink($dir . $file);
                    } catch (Exception $e) {
                    }
                }
            }
            return NULL;
        }

        if (!is_dir($dir)) {
            mkdir($dir, 0777, TRUE);
            chmod($dir, 0777);
        }

        $data = serialize($data);

        return (bool) file_put_contents($dir . $file, $data, LOCK_EX);
    }

    public static function message($file, $path = NULL, $default = NULL) 
    {
        static $messages;
        if (!isset($messages[$file])) {
            $messages[$file] = array();

            if ($files = Rise::find_file('messages', $file)) {
                foreach ($files as $f) {
                    $messages[$file] = Arr::merge($messages[$file], Rise::load($f));
                }
            }
        }

        if ($path === NULL) {
            return $messages[$file];
        } else {
            return Arr::path($messages[$file], $path, $default);
        }
    }

    public static function error_handler($code, $error, $file = NULL, $line = NULL) {
        if (error_reporting() & $code) {
            throw new Exception($error, $code, 0, $file, $line);
        }
        return TRUE;
    }
    public static function shutdown_handler() {
        if (!Rise::$_init) {
            return;
        }

        if (Rise::$caching === TRUE AND Rise::$_files_changed === TRUE) {
            Rise::cache('Rise::find_file()', Rise::$_files);
        }

        if (Rise::$errors AND $error = error_get_last() AND in_array($error['type'], Rise::$shutdown_errors)) {
            ob_get_level() and ob_clean();
            Exception::handler(new Exception($error['message'], $error['type'], 0, $error['file'], $error['line']));
            exit(1);
        }
    }

}
