<?php defined('SYSPATH') or die('No direct script access.');
class Rise_Config {
	protected $_sources = array();
	public function attach(Rise_Config_Source $source, $first = TRUE)
	{
		if ($first === TRUE)
		{
			array_unshift($this->_sources, $source);
		}
		else
		{
			$this->_sources[] = $source;
		}

		return $this;
	}
	public function detach(Rise_Config_Source $source)
	{
		if (($key = array_search($source, $this->_sources)) !== FALSE)
		{
			unset($this->_sources[$key]);
		}

		return $this;
	}
	public function load($group)
	{
		if( ! count($this->_sources))
		{
			throw new Exception('No configuration sources attached');
		}

		if (empty($group))
		{
			throw new Exception("Need to specify a config group");
		}

		if ( ! is_string($group))
		{
			throw new Exception("Config group must be a string");
		}

		if (strpos($group, '.') !== FALSE)
		{
			// Split the config group and path
			list ($group, $path) = explode('.', $group, 2);
		}

		if(isset($this->_groups[$group]))
		{
			if (isset($path))
			{
				return Arr::path($this->_groups[$group], $path, NULL, '.');
			}
			return $this->_groups[$group];
		}

		$config = array();

		$sources = array_reverse($this->_sources);

		foreach ($sources as $source)
		{
			if ($source instanceof Rise_Config_Reader)
			{
				if ($source_config = $source->load($group))
				{
					$config = Arr::merge($config, $source_config);
				}
			}
		}

		$this->_groups[$group] = new Config_Group($this, $group, $config);

		if (isset($path))
		{
			return Arr::path($config, $path, NULL, '.');
		}

		return $this->_groups[$group];
	}
	public function copy($group)
	{
		// Load the configuration group
		$config = $this->load($group);

		foreach($config->as_array() as $key => $value)
		{
			$this->_write_config($group, $key, $value);
		}

		return $this;
	}

	public function _write_config($group, $key, $value)
	{
		foreach($this->_sources as $source)
		{
			if ( ! ($source instanceof Rise_Config_Writer))
			{
				continue;
			}

			
			// Copy each value in the config
			$source->write($group, $key, $value);
		}

		return $this;
	}

}
