<?php defined('SYSPATH') or die('No direct access');
class Rise_Exception extends Exception {

	/**
	 * @var  array  PHP error code => human readable name
	 */
	public static $php_errors = array(
		E_ERROR              => 'Fatal Error',
		E_USER_ERROR         => 'User Error',
		E_PARSE              => 'Parse Error',
		E_WARNING            => 'Warning',
		E_USER_WARNING       => 'User Warning',
		E_STRICT             => 'Strict',
		E_NOTICE             => 'Notice',
		E_RECOVERABLE_ERROR  => 'Recoverable Error',
	);

	public static $error_view = 'kohana/error';

	public static $error_view_content_type = 'text/html';
	public function __construct($message, array $variables = NULL, $code = 0)
	{
		if (defined('E_DEPRECATED'))
		{
			Rise_Exception::$php_errors[E_DEPRECATED] = 'Deprecated';
		}
		$message = __($message, $variables);
		parent::__construct($message, (int) $code);
		$this->code = $code;
	}
	public function __toString()
	{
		return Rise_Exception::text($this);
	}
	public static function handler(Exception $e)
	{
		try
		{
			// Get the exception information
			$type    = get_class($e);
			$code    = $e->getCode();
			$message = $e->getMessage();
			$file    = $e->getFile();
			$line    = $e->getLine();

			// Get the exception backtrace
			$trace = $e->getTrace();

			if ($e instanceof ErrorException)
			{
				if (isset(Rise_Exception::$php_errors[$code]))
				{
					// Use the human-readable error name
					$code = Rise_Exception::$php_errors[$code];
				}

				if (version_compare(PHP_VERSION, '5.3', '<'))
				{
					// Workaround for a bug in ErrorException::getTrace() that exists in
					// all PHP 5.2 versions. @see http://bugs.php.net/bug.php?id=45895
					for ($i = count($trace) - 1; $i > 0; --$i)
					{
						if (isset($trace[$i - 1]['args']))
						{
							// Re-position the args
							$trace[$i]['args'] = $trace[$i - 1]['args'];

							// Remove the args
							unset($trace[$i - 1]['args']);
						}
					}
				}
			}

			// Create a text version of the exception
			$error = Rise_Exception::text($e);

			if (is_object(Rise::$log))
			{
				// Add this exception to the log
				Rise::$log->add(Log::ERROR, $error);

				$strace = Rise_Exception::text($e)."\n--\n" . $e->getTraceAsString();
				Rise::$log->add(Log::STRACE, $strace);

				// Make sure the logs are written
				Rise::$log->write();
			}

			if (Rise::$is_cli)
			{
				// Just display the text of the exception
				echo "\n{$error}\n";

				exit(1);
			}

			if ( ! headers_sent())
			{
				// Make sure the proper http header is sent
				$http_header_status = ($e instanceof HTTP_Exception) ? $code : 500;

				header('Content-Type: '.Rise_Exception::$error_view_content_type.'; charset='.Rise::$charset, TRUE, $http_header_status);
			}

			if (Request::$current !== NULL AND Request::current()->is_ajax() === TRUE)
			{
				// Just display the text of the exception
				echo "\n{$error}\n";

				exit(1);
			}

			// Start an output buffer
			ob_start();

			// Include the exception HTML
			if ($view_file = Rise::find_file('views', Rise_Exception::$error_view))
			{
				include $view_file;
			}
			else
			{
				throw new Rise_Exception('Error view file does not exist: views/:file', array(
					':file' => Rise_Exception::$error_view,
				));
			}

			// Display the contents of the output buffer
			echo ob_get_clean();

			exit(1);
		}
		catch (Exception $e)
		{
			// Clean the output buffer if one exists
			ob_get_level() and ob_clean();

			// Display the exception text
			echo Rise_Exception::text($e), "\n";

			// Exit with an error status
			exit(1);
		}
	}
	public static function text(Exception $e)
	{
		return sprintf('%s [ %s ]: %s ~ %s [ %d ]',
			get_class($e), $e->getCode(), strip_tags($e->getMessage()), Debug::path($e->getFile()), $e->getLine());
	}

}
