<?php

/**
 * @author Sergey Jdanuk aka Shadoof <jdanuk at gmail.com>
 */
class Screen 
{
    public static $overrideFunctions = array(
        'get_tv_playback_url',
        'get_tv_stream_url',
        'get_day_epg'
    );
    static public $screens = array();
    
    function __construct() {
        self::add($this);
        foreach(self::$overrideFunctions as $func){
            self::$overrideFunctions[$func] = array(get_class($this),$func);
        }
    }

    static function add( $object )
    {
        $obj = self::prepare($object);
        self::$screens[] = $obj;
        return $obj;
    }
    static function prepare( $object )
    {
        return (object)array('handler'=>$object);
    }
    
    static function explode($delimeter, $call)
    {
        $tmp = explode('->',$call);
        $name = $tmp[0];
        $method = NULL;
        $params = array();
        
        if(count($tmp)>1)
        {
            $params = explode('/',$tmp[1]);
            if(count($params))
            {
                $method = $params[0];
                unset($params[0]);
            }
        }
        return array
        (
            'name'=>$name,
            'method'=>$method,
            'params'=>$params
        );
    }
    
    static function find( $call )
    {
        $classInfo = self::explode('->',$call);
        foreach( self::$screens as $i)
        {
            if( get_class( $i->handler ) == $classInfo['name'] )
            {
                return array(
                    'object'=>$i->handler,
                    'method'=>$classInfo['method'],
                    'params'=>$classInfo['params'],
                );
            }
        }
        return FALSE;
    }
    
    static function findOrCreate($call)
    {
        $screen = Screen::find($call);
        if( $screen === FALSE )
        {
            $classInfo = Screen::explode('->', $call);
            $className = $classInfo['name'];

            // Если ничего не нашли, пытаемся создавать новый объект вызываемого класса
            // 
            // Чтобы cоздаваевый класс попал в список screens, 
            // он должен быть наследован от класса Screen
            new $className();

            // Пытаемся еще раз найти объект
            $screen = Screen::find($call);
            if( $screen === FALSE )
            { 
                throw new Exception("The class $className isn't extended of Screen");
            }
        }
        return $screen;
    }
    
    
    final static function _folder_view($media_url)
    {
        if( $media_url == 'dummy' )
            return array('');
        $mediaUrl = Media_Url::decode($media_url);
        if( (isset($mediaUrl->selected_media_url) && $mediaUrl->selected_media_url == 'dummy' ) || $mediaUrl->media_url == 'dummy' )
            return array('');
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        $method = $screen['method'];
        $params = (array)$mediaUrl->params;
        return call_user_func_array(array($object,$method),$params);
    }
    final static function _next_folder_view($media_url)
    {
        if( $media_url == 'dummy' )
            return array('');
        $mediaUrl = Media_Url::decode($media_url);
        if( $mediaUrl->selected_media_url == 'dummy' || $mediaUrl->media_url == 'dummy' )
            return array('');
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        $method = $screen['method'];
        $params = (array)$mediaUrl->params;
        return call_user_func_array(array($object,$method),$params);
    }
    
    final static function _vod_info($media_url)
    {
        if( $media_url == 'dummy' )
            return array('');
        $mediaUrl = Media_Url::decode($media_url);
        if( (isset($mediaUrl->selected_media_url) && $mediaUrl->selected_media_url == 'dummy') || $mediaUrl->media_url == 'dummy' )
            return array('');
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        $method = $screen['method'];
        $params = (array)$mediaUrl->params;
        return call_user_func_array(array($object,$method),$params);
    }

    final static function _tv_info($media_url)
    {
        if( $media_url == 'dummy' )
            return array('');
        $mediaUrl = Media_Url::decode($media_url);
        if( (isset($mediaUrl->selected_media_url) && $mediaUrl->selected_media_url == 'dummy') || $mediaUrl->media_url == 'dummy' )
            return array('');
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        $method = $screen['method'];
        $params = (array)$mediaUrl->params;
        return call_user_func_array(array($object,$method),$params);
    }
    
    final static function _regular_folder_items($media_url, $from_ndx)
    {
        if( $media_url == 'dummy' )
            return array('');
        $mediaUrl = Media_Url::decode($media_url);
        if( (isset($mediaUrl->selected_media_url) && $mediaUrl->selected_media_url == 'dummy' ) || $mediaUrl->media_url == 'dummy' )
            return array('');
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        $method = $screen['method'];
        $params = (array)$mediaUrl->params;
        $params = array_merge(array($from_ndx), $params);
        return call_user_func_array(array($object,$method.'Regular'),$params);
    }
    
    final static function _handle_user_input(&$arg)
    {
        $mediaUrl = "";
        if( isset($arg->control_id) && ( $arg->control_id == 'search' || $arg->control_id == 'searchResult' ) )
        {
            $mediaUrl = Media_Url::decode($arg->parent_media_url);
        }
        else if(isset($arg->selected_media_url))
        {
            if( $arg->selected_media_url == 'dummy' )
                return array('');

            $mediaUrl = Media_Url::decode($arg->selected_media_url);
        }
        if( !isset($mediaUrl->media_url) || empty($mediaUrl->media_url))
        {
            print_rex($arg);
            throw new Exception("Media url is empty - screen.php:".__LINE__);
        }
        $screen = self::findOrCreate($mediaUrl->media_url);
        $object = $screen['object'];
        return $object->input($arg);
    }

    final static function _get_tv_playback_url($channel_id, $archive_ts, $protect_code){
        return call_user_func_array(self::$overrideFunctions['get_tv_playback_url'],func_get_args());
    }
    final static function _get_day_epg($channel_id, $day_start_ts){
        return call_user_func_array(self::$overrideFunctions['get_day_epg'],func_get_args());
    }
    final static function _get_tv_stream_url($media_url){
        return call_user_func_array(self::$overrideFunctions['get_tv_stream_url'],func_get_args());
    }
}
?>
